import math
from adap_gl import AdapGL as agl
#from calcroot import Newton
#from Calc_z_NormalDistri import calc_z

def sqr( x ):
    return x * x

class C_chi2_tan_kernel:
    def __init__(self, df):
        self.df = df

    def v(self, theta):
        x = math.tan(theta)
        v = pow(x, (self.df/2.0) - 1.0) * math.exp(-0.5 * x)
        return v / sqr(math.cos(theta))
                
class C_chi2_tan_density:
    def __init__(self, df):
        self.df = df
        chi2_kernel = C_chi2_tan_kernel(self.df)
        gl = agl()
        v = gl.integral( 0.0, math.pi/2.0, chi2_kernel)
        self.c = 1.0 / v

    def v(self, theta):
        x = math.tan(theta)
        v = pow(x, (self.df/2.0) - 1.0) * math.exp(-0.5 * x)
        return self.c * v / sqr(math.cos(theta))
        
class C_cum_chi2:
    def __init__(self, df):
        self.df = df
        self.chi2_tan = C_chi2_tan_density(self.df)
        self.gl = agl()

    def v( self, x ):
        chi2_cum = self.gl.integral(0.0, math.atan(x), self.chi2_tan)
        return chi2_cum


while True:
    try:
        s = input("df = ")
        df = float(s)
        cum_chi2 = C_cum_chi2(df)
        s = input("chi^2 = ")
        chi2 = float(s)
        p = cum_chi2.v(chi2)
        print("p = %f   chi^2 = %f"%(p, chi2))
    except:
        break
    

